@@ -1,4 +1,4 @@
-/** -*- mode: c++ ; c-basic-offset: 3 -*-
+/** -*- mode: c++ ; c-basic-offset: 2 -*-
  * @file   FilterThread.h
  * @author Sebastien Fourey
  * @date   July 2010
@@ -49,9 +49,9 @@
 
 #include <QThread>
 #include "Common.h"
+#include "gmic.h"
 #include "CImg.h"
 #include "CriticalRef.h"
-#include "gmic.h"
 class ImageSource;
 class QMutex;
 class QImage;
@@ -61,12 +61,14 @@ class FilterThread : public QThread {
   Q_OBJECT
 public:
 
-  enum PreviewMode { Full, TopHalf, LeftHalf, BottomHalf, RightHalf, Original };
+  enum PreviewMode { Full, TopHalf, LeftHalf, BottomHalf, RightHalf, DuplicateVertical, DuplicateHorizontal, Original };
 
   FilterThread( ImageSource & webcam,
                 const QString & command,
-                QImage * outputImage,
-                QMutex * imageMutex,
+                QImage * outputImageA,
+                QMutex * imageMutexA,
+                QImage * outputImageB,
+                QMutex * imageMutexB,
                 PreviewMode previewMode,
                 int frameSkip,
                 int fps,
@@ -99,8 +101,10 @@ private:
   QString _command;
   CriticalRef<QString> _arguments;
   bool _commandUpdated;
-  QImage * _outputImage;
-  QMutex * _imageMutex;
+  QImage * _outputImageA;
+  QMutex * _imageMutexA;
+  QImage * _outputImageB;
+  QMutex * _imageMutexB;
   QSemaphore * _blockingSemaphore;
   PreviewMode _previewMode;
   int _frameSkip;
