@@ -1,4 +1,4 @@
-/** -*- mode: c++ ; c-basic-offset: 3 -*-
+/** -*- mode: c++ ; c-basic-offset: 2 -*-
  * @file   FilterThread.cpp
  * @author Sebastien Fourey
  * @date   July 2010
@@ -60,8 +60,10 @@ using namespace cimg_library;
 
 FilterThread::FilterThread(ImageSource & imageSource,
                            const QString & command,
-                           QImage * outputImage,
-                           QMutex * imageMutex,
+                           QImage * outputImageA,
+                           QMutex * imageMutexA,
+                           QImage * outputImageB,
+                           QMutex * imageMutexB,
                            PreviewMode previewMode,
                            int frameSkip,
                            int fps,
@@ -69,8 +71,10 @@ FilterThread::FilterThread(ImageSource & imageSource,
   : _imageSource( imageSource ),
     _arguments(new QString("")),
     _commandUpdated(true),
-    _outputImage( outputImage ),
-    _imageMutex( imageMutex ),
+    _outputImageA( outputImageA ),
+    _imageMutexA( imageMutexA ),
+    _outputImageB( outputImageB ),
+    _imageMutexB( imageMutexB ),
     _blockingSemaphore( blockingSemaphore ),
     _previewMode( previewMode ),
     _frameSkip( frameSkip ),
@@ -81,8 +85,11 @@ FilterThread::FilterThread(ImageSource & imageSource,
     _gmic_images(),
     _gmic(0)
 {
-   setCommand(command);
-   setFPS(fps);
+  setCommand(command);
+  setFPS(fps);
+#ifdef _IS_MACOS_
+  setStackSize(8*1024*1024);
+#endif
 }
 
 FilterThread::~FilterThread()
@@ -123,7 +130,9 @@ FilterThread::setFPS(int fps)
 {
   _fps = fps;
   _frameInterval = 0;
-  if ( fps > 0 ) _frameInterval = 1000/fps;
+  if ( fps > 0 ) {
+    _frameInterval = 1000/fps;
+  }
 }
 
 void
@@ -167,10 +176,10 @@ FilterThread::run()
       timeMeasure.restart();
 
       if ( _commandUpdated ) {
-         delete _gmic;
-         QString c = QString("foo: -skip $\"*\" ") + _command;
-         _gmic = new gmic( "", c.toLatin1().constData() );
-         _commandUpdated = false;
+        delete _gmic;
+        QString c = QString("foo: -skip $\"*\" ") + _command;
+        _gmic = new gmic( "", c.toLatin1().constData() );
+        _commandUpdated = false;
       }
 
       _gmic->run("-v -");
@@ -187,7 +196,7 @@ FilterThread::run()
       _arguments.unlock();
       c += call;
 
-     // QSHOW( call );
+      // QSHOW( call );
 
       _gmic->run( c.toLatin1().constData(),_gmic_images,_gmic_images_names );
       lastCommandDuration = timeMeasure.elapsed();
@@ -195,47 +204,85 @@ FilterThread::run()
       switch ( _previewMode ) {
       case Full:
         if (_gmic_images && _gmic_images[0]) {
-          _imageMutex->lock();
           QSize size( _gmic_images[0].width(), _gmic_images[0].height() );
-          if ( _outputImage->size() != size ) {
-            *_outputImage = QImage( size, QImage::Format_RGB888 );
+          _imageMutexA->lock();
+          if ( _outputImageA->size() != size ) {
+            *_outputImageA = QImage( size, QImage::Format_RGB888 );
+          }
+          _imageMutexA->unlock();
+          ImageConverter::convert( _gmic_images[0], _outputImageA );
+          if ( _outputImageB ) {
+            _imageMutexB->lock();
+            if ( _outputImageB->size() != size ) {
+              *_outputImageB = QImage( size, QImage::Format_RGB888 );
+            }
+            _imageMutexB->unlock();
+            ImageConverter::convert( _gmic_images[0], _outputImageB );
           }
-          _imageMutex->unlock();
-          ImageConverter::convert( _gmic_images[0], _outputImage );
         }
         break;
       case Original: {
-        _imageMutex->lock();
+        _imageMutexA->lock();
         QSize size( _imageSource.width(), _imageSource.height() );
-        if ( _outputImage->size() != size ) {
-          *_outputImage = QImage( size, QImage::Format_RGB888 );
+        if ( _outputImageA->size() != size ) {
+          *_outputImageA = QImage( size, QImage::Format_RGB888 );
         }
-        _imageMutex->unlock();
-        ImageConverter::convert( _imageSource.image(), _outputImage );
+        _imageMutexA->unlock();
+        if ( _outputImageB ) {
+          if ( _outputImageB->size() != size ) {
+            *_outputImageB = QImage( size, QImage::Format_RGB888 );
+          }
+          _imageMutexB->unlock();
+        }
+        ImageConverter::convert( _imageSource.image(), _outputImageA );
+        ImageConverter::convert( _imageSource.image(), _outputImageB );
       }
         break;
       case LeftHalf:
-        ImageConverter::merge( _imageSource.image(),_gmic_images[0], _outputImage,
-            _imageMutex, ImageConverter::MergeLeft );
+        ImageConverter::merge( _imageSource.image(),_gmic_images[0],
+            _outputImageA, _imageMutexA, ImageConverter::MergeLeft );
+        ImageConverter::merge( _imageSource.image(),_gmic_images[0],
+            _outputImageB, _imageMutexB, ImageConverter::MergeLeft );
         break;
       case TopHalf:
-        ImageConverter::merge( _imageSource.image(), _gmic_images[0], _outputImage,
-            _imageMutex, ImageConverter::MergeTop );
+        ImageConverter::merge( _imageSource.image(), _gmic_images[0],
+            _outputImageA, _imageMutexA, ImageConverter::MergeTop );
+        ImageConverter::merge( _imageSource.image(), _gmic_images[0],
+            _outputImageB, _imageMutexB, ImageConverter::MergeTop );
         break;
       case BottomHalf:
-        ImageConverter::merge( _imageSource.image(), _gmic_images[0], _outputImage,
-            _imageMutex, ImageConverter::MergeBottom );
+        ImageConverter::merge( _imageSource.image(), _gmic_images[0],
+            _outputImageA, _imageMutexA, ImageConverter::MergeBottom );
+        ImageConverter::merge( _imageSource.image(), _gmic_images[0],
+            _outputImageB, _imageMutexB, ImageConverter::MergeBottom );
         break;
       case RightHalf:
-        ImageConverter::merge( _imageSource.image(), _gmic_images[0], _outputImage,
-            _imageMutex, ImageConverter::MergeRight );
+        ImageConverter::merge( _imageSource.image(), _gmic_images[0],
+            _outputImageA, _imageMutexB, ImageConverter::MergeRight );
+        ImageConverter::merge( _imageSource.image(), _gmic_images[0],
+            _outputImageB, _imageMutexB, ImageConverter::MergeRight );
+        break;
+      case DuplicateHorizontal:
+        ImageConverter::merge( _imageSource.image(), _gmic_images[0],
+            _outputImageA, _imageMutexA, ImageConverter::DuplicateHorizontal);
+        ImageConverter::merge( _imageSource.image(), _gmic_images[0],
+            _outputImageB, _imageMutexB, ImageConverter::DuplicateHorizontal);
+        break;
+      case DuplicateVertical:
+        ImageConverter::merge( _imageSource.image(), _gmic_images[0],
+            _outputImageA, _imageMutexA, ImageConverter::DuplicateVertical);
+        ImageConverter::merge( _imageSource.image(), _gmic_images[0],
+            _outputImageB, _imageMutexB, ImageConverter::DuplicateVertical);
         break;
       default:
-        _outputImage->fill( QColor(255,255,255).rgb() );
+        _outputImageA->fill( QColor(255,255,255).rgb() );
+        if ( _outputImageB ) {
+          _outputImageB->fill( QColor(255,255,255).rgb() );
+        }
         break;
       }
 
-    } catch (gmic_exception &e) {
+    } catch (gmic_exception & e) {
       CImg<unsigned char> src( reinterpret_cast<unsigned char*>(_imageSource.image()->imageData),
                                3, _imageSource.width(), _imageSource.height(), 1, true );
       _gmic_images = src.get_permute_axes("yzcx");
@@ -248,16 +295,23 @@ FilterThread::run()
       }
       std::cerr << e.what() << std::endl;
       QSize size( _imageSource.image()->width, _imageSource.image()->height );
-      if ( _outputImage->size() != size ) {
-        _imageMutex->lock();
-        *_outputImage = QImage( size, QImage::Format_RGB888 );
-        _imageMutex->unlock();
+      if ( _outputImageA->size() != size ) {
+        _imageMutexA->lock();
+        *_outputImageA = QImage( size, QImage::Format_RGB888 );
+        _imageMutexA->unlock();
       }
-      ImageConverter::convert( _gmic_images[0], _outputImage );
+      if ( _outputImageB->size() != size ) {
+        _imageMutexB->lock();
+        *_outputImageB = QImage( size, QImage::Format_RGB888 );
+        _imageMutexB->unlock();
+      }
+      ImageConverter::convert( _gmic_images[0], _outputImageA );
+      ImageConverter::convert( _gmic_images[0], _outputImageB );
     }
     emit imageAvailable();
-    if ( !_fps && _blockingSemaphore )
+    if ( !_fps && _blockingSemaphore ) {
       _blockingSemaphore->acquire(_blockingSemaphore->available()+1);
+    }
   }
 }
 
@@ -270,8 +324,8 @@ FilterThread::setCommand( const QString & command )
 {
   QByteArray str = command.toLatin1();
   _command = str.constData();
-  _command.replace( "@{!,x}", "$_x" )
-        .replace( "@{!,y}", "$_y" )
-        .replace( "@{!,b}", "$_b" );
+  _command.replace( "{*,x}", "$_x" )
+      .replace( "{*,y}", "$_y" )
+      .replace( "{*,b}", "$_b" );
   _commandUpdated = true;
 }
