@@ -1,4 +1,4 @@
-/** -*- mode: c++ ; c-basic-offset: 3 -*-
+/** -*- mode: c++ ; c-basic-offset: 2 -*-
  * @file   ImageConverter.cpp
  * @author Sebastien Fourey
  * @date   Jul 2010
@@ -55,6 +55,7 @@ IplImage * ImageConverter::_image = 0;
 
 void ImageConverter::convert( const IplImage * in, QImage * out )
 {
+  if ( !in || !out ) return;
   assert(in->depth== IPL_DEPTH_8U);
   assert(in->nChannels == 3);
   const unsigned int w3 = 3 * in->width;
@@ -80,6 +81,7 @@ void ImageConverter::convert( const IplImage * in, QImage * out )
 
 void ImageConverter::convert(const QImage & in, IplImage **out)
 {
+  if ( !out ) return;
   *out = cvCreateImage(cvSize(in.width(),in.height()),IPL_DEPTH_8U,3);
   const unsigned int w3 = 3 * in.width();
   const unsigned int qiOffset = (w3%4)?(4-(w3%4)):0;
@@ -103,6 +105,7 @@ void ImageConverter::convert(const QImage & in, IplImage **out)
 
 void ImageConverter::convert( const cimg_library::CImg<float> & in, QImage * out )
 {
+  if ( !out ) return;
   const int spectrum = in.spectrum();
   unsigned char * dst = out->scanLine(0);
   const float *srcR = in.data( 0,0,0,0 );
@@ -129,6 +132,7 @@ void ImageConverter::merge( IplImage * iplImage,
                             QMutex * imageMutex,
                             MergeDirection direction )
 {
+  if ( !iplImage || !out ) return;
   IplImage * cameraImage = iplImage;
   if ( iplImage->width != cimgImage.width() || iplImage->height != cimgImage.height() ) {
     if ( _image )
@@ -156,8 +160,13 @@ void ImageConverter::merge( IplImage * iplImage,
   case MergeRight:
     mergeRight( cameraImage, cimgImage, out );
     break;
+  case DuplicateVertical:
+    mergeBottom(cameraImage, cimgImage, out, false );
+    break;
+  case DuplicateHorizontal:
+    mergeRight(cameraImage, cimgImage, out, false );
+    break;
   }
-
 }
 
 void ImageConverter::mergeTop( IplImage * iplImage,
@@ -246,9 +255,10 @@ void ImageConverter::mergeLeft( IplImage * iplImage,
   }
 }
 
-void ImageConverter::mergeBottom( IplImage * iplImage,
-                                  const cimg_library::CImg<float> & cimgImage,
-                                  QImage * out )
+void ImageConverter::mergeBottom(IplImage * iplImage,
+                                 const cimg_library::CImg<float> & cimgImage,
+                                 QImage * out ,
+                                 bool shift)
 {
   const int height = iplImage->height;
   const int width = iplImage->width;
@@ -257,7 +267,6 @@ void ImageConverter::mergeBottom( IplImage * iplImage,
   const unsigned int qiOffset = ((width*3)%4)?(4-((width*3)%4)):0;
   const unsigned int iplOffset = iplImage->widthStep - 3 * width;
 
-
   // Copy from iplImage
   unsigned char * srcIpl = reinterpret_cast<unsigned char *>( iplImage->imageData );
   unsigned int lines = height/2;
@@ -274,7 +283,7 @@ void ImageConverter::mergeBottom( IplImage * iplImage,
   }
 
   // Copy from cimgImage
-  const unsigned int cimgOffset = width * (height/2);
+  const unsigned int cimgOffset = (shift ? (width * (height/2)) : 0);
   const int spectrum = cimgImage.spectrum();
   const float *srcR = cimgImage.data( 0,0,0,0 ) + cimgOffset;
   const float *srcG = cimgImage.data( 0,0,0,(spectrum>=2)?1:0 ) + cimgOffset;
@@ -291,18 +300,19 @@ void ImageConverter::mergeBottom( IplImage * iplImage,
   }
 }
 
-void ImageConverter::mergeRight( IplImage * iplImage,
-                                 const cimg_library::CImg<float> & cimgImage,
-                                 QImage * out )
+void ImageConverter::mergeRight(IplImage * iplImage,
+                                const cimg_library::CImg<float> & cimgImage,
+                                QImage * out ,
+                                bool shift)
 {
   const int width = iplImage->width;
   const unsigned int firstHalf = width/2;
   const unsigned int secondHalf = width - width/2;
 
   const int spectrum = cimgImage.spectrum();
-  const float *srcR = cimgImage.data( 0,0,0,0 ) + firstHalf;
-  const float *srcG = cimgImage.data( 0,0,0,(spectrum>=2)?1:0 ) + firstHalf;
-  const float *srcB = cimgImage.data( 0,0,0,(spectrum>=3)?2:0 ) + firstHalf;
+  const float *srcR = cimgImage.data( 0,0,0,0 ) + (shift?firstHalf:0);
+  const float *srcG = cimgImage.data( 0,0,0,(spectrum>=2)?1:0 ) + (shift?firstHalf:0);
+  const float *srcB = cimgImage.data( 0,0,0,(spectrum>=3)?2:0 ) + (shift?firstHalf:0);
 
   unsigned char * srcIpl = reinterpret_cast<unsigned char *>( iplImage->imageData );
   const unsigned int iplOffset = iplImage->widthStep - 3 * width;
@@ -333,7 +343,6 @@ void ImageConverter::mergeRight( IplImage * iplImage,
     srcR += firstHalf;
     srcG += firstHalf;
     srcB += firstHalf;
-
     dst += qiOffset;
   }
 }
